<?php
/**
 * Plugin Name: KICT Payment Gateway
 * Plugin URI: https://www.k-ict.org/Wordpress/KICT_Payment_Gateway/KICT_Payment_Gateway.zip
 * Description: Internet Banking Online Payment Gateway using Malaysia saving and current account via FPX | Developed by Konsortium ICT Pantai Timur (formerly known as K-ICT). Contact <a href="https://www.k-ict.org/v4/contact-us" target="KICTwindow">sales@k-ict.org</a> for further details.
 * Author: K-ICT
 * Author URI: https://www.k-ict.org/
 * Version: 1.1
 */

# Function to display 'Setting' on the 'WordPress Installed plugins' page.
function KPG_plugin_settings_link($links)
{
	$settings_link='<a href="admin.php?page=wc-settings&tab=checkout&section=KPG">Settings</a>';
	array_unshift($links,$settings_link);
	return $links;
}

$plugin = plugin_basename(__FILE__);

# Add that 'Setting' on the appropriate page.
add_filter("plugin_action_links_$plugin",'KPG_plugin_settings_link');

# Add the plugin.
add_action('plugins_loaded','KPG_gateway_load',0);

function KPG_gateway_load()
{ # Function KPG_gateway_load start.

	# Add the plugin to the list of gateways.
	add_filter('woocommerce_payment_gateways','KPG_add_gateway');

	function KPG_add_gateway($methods)
	{ # Method to add to the list of gateway start.
		$methods[]='KPG_gateway';
		return $methods;
	} # Method to add to the list of gateway end.

	class KPG_gateway extends WC_Payment_Gateway
	{ # Class KPG_gateway start.
		public function __construct()
		{ # The class constructor start.
			global $woocommerce;
			$this->id='kpg';
			$this->icon=plugins_url('images/FPX.jpg',__FILE__);
			$this->order_button_text=__( 'Proceed to FPX','KPG');
			$this->has_fields=false;
			$this->method_title=__('KPG','KPG');

			# Load the form fields.
			$this->init_form_fields();

			# Load the settings.
			$this->init_settings();

			# Define user setting variables.
			$this->title='FPX';
			$this->description='Pay using your saving or current Malaysia Internet Banking account.';
			$this->portal_key=$this->settings['portal_key'];

			# Add the order receipt page.
			add_action('woocommerce_receipt_'.$this->id,array(&$this,'receipt_page'));

			# Save setting configuration.
			add_action('woocommerce_update_options_payment_gateways_'.$this->id,array($this,'process_admin_options'));

			# Listen to the response from KPG.
			if(isset($_POST['portalKey']) && isset($_POST['orderNo']) && isset($_POST['orderAmount']) && isset($_POST['buyerName']) && isset($_POST['buyerEmail']) && isset($_POST['buyerTel']) && isset($_POST['orderDescription']) && isset($_POST['txnId']) && isset($_POST['txnEx']) && isset($_POST['txnStatus']))
			$this->check_response();

			# Check whether KPG Portal key is empty or not.
			$this->portal_key==''?add_action('admin_notices',array(&$this,'portal_key_missing_message')):'';
		} # The class constructor end.

		public function admin_options()
		{ # Function admin_options start.
		?>
			<h3><?php _e('KICT Payment Gateway','KPG'); ?></h3>
			<div align="justify" style="font-size:18px; padding-bottom:10px;">
			<img alt="<?php echo $this->title; ?>" title="<?php echo $this->title; ?>" src="<?php echo plugins_url('images/FPX.jpg',__FILE__); ?>">
			<br>FPX is the Online Payment Gateway for saving and current Internet Banking account in Malaysia,that is developed by MyClear (subsidiary of Bank Negara Malaysia). K-ICT has developed KICT Payment Gateway to simplify the FPX for multiple sellers purpose. The payment will be transmitted to the appropriate seller based on the provided portal key below. There is only FPX in KICT Payment Gateway for this version. Other payment options will be added in the future version.
			</div>
			<div style="background:#333;color:#fff;padding:10px;">Please consider to use <font style="color:#ff0;">Post name</font> for the <font style="color:#ff0;">Permalink Settings</font> since this plugin will only work with SEO-friendly setting.</div>
			<table class="form-table">
			<?php $this->generate_settings_html(); ?>
			</table>
			<?php
		} # Function admin_options end.

		public function init_form_fields()
		{ # Admin form initialization for the settings start.
			$this->form_fields=array(
			'enabled'=>array(
			'title'=>__('Enable/Disable','KPG'),
			'type'=>'checkbox',
			'label'=>__('Enable KICT Payment Gateway','KPG'),
			'default'=>'yes'
			),
			'portal_key'=>array(
			'title'=>__('Portal Key','KPG'),
			'type'=>'text',
			'placeholder'=>'',
			'description'=>__('Copy your Portal key from <a href="https://www.k-ict.org/kpg/?modules=portal" target="KPGwindow">KPG</a> and paste in the field.<div align="left" style="background:#358;color:#fff;padding:10px;">Please config the <font style="color:#ff0;">KPG Receipt URL</font> as <font style="color:#ff0;">'.get_permalink(woocommerce_get_page_id('checkout')).'order-pay/receipt/</font></div>','KPG'),
			'default'=>''
			),
			);
		} # Admin form initialization for the settings end.

		public function generate_form($order_no)
		{ # The order form for KPG submission start.
			$this->init_settings();
			$order=new WC_Order($order_no);

			# Change the requested order status to pending payment before initiating FPX transaction.
			if($order->status!='pending')
			$order->update_status('pending');

			?>
			<form id="KPGpaymentForm" method="POST" action="https://www.k-ict.org/kpg/payment.php">
			<?php
			$payment_data_array=array(
			'portal_key'=>$this->portal_key,
			'order_no'=>$order_no,
			'amount'=>$order->order_total,
			'buyer_name'=>$order->billing_first_name.' '.$order->billing_last_name,
			'buyer_email'=>$order->billing_email,
			'buyer_tel'=>$order->billing_phone,
			'description'=>'Order '.$order_no
			);

			foreach($payment_data_array as $payment_data_array_variable=>$payment_data_array_value)
			{ # Loop each required data to be submitted start.
			?>
			<input name="<?php echo $payment_data_array_variable; ?>" type="hidden" value="<?php echo $payment_data_array_value; ?>">
			<?php
			} # Loop each required data to be submitted end.
			?>
			<input type="button" class="button" value="Continue" onclick="document.getElementById('KPGpaymentForm').submit();">
			<input type="button" class="button" value="Cancel" onclick="location.href='<?php echo $order->get_cancel_order_url(); ?>'">
			<script>
			document.getElementById('KPGpaymentForm').submit(); // Auto-click Continue button.
			</script>
			</form>
			<?php
		} # The order form for KPG submission end.

		public function process_payment($order_no)
		{ # Function to process the payment start.
			$order=new WC_Order($order_no);
			return array('result'=>'success','redirect'=>$order->get_checkout_payment_url(true));
		} # Function to process the payment end.

		public function receipt_page($order)
		{ # Function that generate the order form start.
			echo $this->generate_form($order);
		} # Function that generate the order form end.

		function check_response()
		{ # Function to check the responses from KPG start.
			$this->init_settings();
			$db_portal_key=$this->settings['portal_key'];
			$order=new WC_Order(addslashes($_POST['orderNo']));
			global $woocommerce;

			# Check for the order stored in WooCommerce database start.
			$order_db_no=$order->id;
			$order_db_amount=$order->total;
			$order_db_buyer_name=$order->billing_first_name.' '.$order->billing_last_name;
			$order_db_buyer_email=$order->billing_email;
			$order_db_buyer_tel=$order->billing_phone;
			$order_db_status=$order->status;
			# Check for the order stored in WooCommerce database end.

			# Check for the order POSTed from KPG start.
			$order_no_post=urldecode($_POST['orderNo']);
			$portal_key=urldecode($_POST['portalKey']);
			$order_amount=urldecode($_POST['orderAmount']);
			$order_buyer_name=urldecode($_POST['buyerName']);
			$order_buyer_email=urldecode($_POST['buyerEmail']);
			$order_buyer_tel=urldecode($_POST['buyerTel']);
			$order_description=urldecode($_POST['orderDescription']);
			$transaction_fpx=urldecode($_POST['txnId']);
			$transaction_kpg=urldecode($_POST['txnEx']);
			$transaction_status=urldecode($_POST['txnStatus']);
			$transaction_bank_name=urldecode($_POST['txnBankName']);
			$transaction_seller_name=urldecode($_POST['txnSellerName']);
			$transaction_time=urldecode($_POST['txnExTime']);
			$transaction_currency=urldecode($_POST['txnCurrency']);
			$transaction_status_description=urldecode($_POST['txnStatusDescr']);
			# Check for the order POSTed from KPG end.

			$hash_response=hash('sha512',"$portal_key|$order_no_post");
			$hash_db=hash('sha512',"$db_portal_key|$order_db_no");

			if($hash_response==$hash_db)
			{ # Do only proceed on the matched hash responses and hash in the database start.
				if($order->status=='pending')
				{ # Permit to process order with pending payment status start.
					if($transaction_status=='Successful')
					{ # Do action on successful transaction start.
						if($order->status!='processing')
						{ # Do only update order to processing if the current status is not processing start.
							$order->add_order_note('Thank you for your payment.<br>FPX Trx. ID : <a href="https://www.k-ict.org/kpg/receipt.php?txnId='.$transaction_fpx.'&txnEx='.$transaction_kpg.'" title="Click here to view transaction receipt at KPG" target="KPGwindow">'.$transaction_fpx.'</a><br>KPG Trx. ID : <a href="https://www.k-ict.org/kpg/receipt.php?txnId='.$transaction_fpx.'&txnEx='.$transaction_kpg.'" title="Click here to view transaction receipt at KPG" target="KPGwindow">'.$transaction_kpg.'</a>');
							$order->payment_complete();
							$order->update_status('processing');
							$woocommerce->cart->empty_cart();
						} # Do only update order to processing if the current status is not processing end.
					} # Do action on successful transaction end.
					else
					{ # Do action on unsuccessful transaction start.
						if($order->status!='failed')
						{ # Do only update order to failed if the current status is not failed start.
							$order->add_order_note('Sorry,your payment is unsuccessful.<br>Description : '.$transaction_status_description.'<br>FPX Trx. ID : <a href="https://www.k-ict.org/kpg/receipt.php?txnId='.$transaction_fpx.'&txnEx='.$transaction_kpg.'" title="Click here to view transaction receipt at KPG" target="KPGwindow">'.$transaction_fpx.'</a><br>KPG Trx. ID : <a href="https://www.k-ict.org/kpg/receipt.php?txnId='.$transaction_fpx.'&txnEx='.$transaction_kpg.'" title="Click here to view transaction receipt at KPG" target="KPGwindow">'.$transaction_kpg.'</a><br><a href="'.$order->get_checkout_payment_url().'">Click here</a> to make another payment.');
							$order->update_status('failed');
						} # Do only update order to failed if the current status is not failed end.
					} # Do action on unsuccessful transaction start.
					?>
				<style type="text/css">
				.transactionSummaryDiv
				{
				background: #eee;
				font-size: 14px;
				border-radius: 8px;
				padding: 8px;
				font-weight: normal;
				}
				.transactionSummaryTitle
				{
				color: #0087cb;
				font-size: 20px;
				text-decoration: underline;
				font-weight: normal;
				}
				.transactionContainerContents
				{
				background: #fff;
				padding: 20px;
				}

				input[type=button]:hover
				{
				opacity:0.8;
				}
				input[type=button]
				{
				color: #fff;
				text-align: center;
				background-color: #088;
				display: inline-block;
				cursor: pointer;
				color: #fff;
				font-family: Arial;
				font-size: 10px;
				font-weight: bold;
				padding: 8px 20px 8px 20px;
				text-decoration: none;
				margin: 2px;
				border: 1px solid #088;
				border-radius: 5px;
				}
				</style>

				<script type="text/javascript">
				function CallPrint(strid,windowTitleName)
				{
				var printContent=document.getElementById(strid);
				if(document.getElementsByClassName('expand'))
				document.getElementsByClassName('expand').click();
				var windowUrl=windowTitleName;
				var uniqueName=new Date();
				var windowName="FPX Transaction Receipt";

				var printWindow=window.open(windowUrl,windowName,'left=200,top=50,width=800,height=600');

				printWindow.document.write('<html>\n');
				printWindow.document.write('<head>\n');
				printWindow.document.write('<title>'+windowUrl+' - '+windowName+'</title>\n');

				if(navigator.userAgent.toLowerCase().indexOf("chrome") > -1)
				{

				}
				else
				{
				}

				printWindow.document.write('<script>\n');

				if(navigator.userAgent.toLowerCase().indexOf("chrome") > -1)
				{
				}

				printWindow.document.write('function winPrint()\n');
				printWindow.document.write('{\n');
				printWindow.document.write('window.focus();\n');


				if(navigator.userAgent.toLowerCase().indexOf("chrome") > -1)
				{
				printWindow.document.write('printChrome();\n');
				}
				else
				{
				printWindow.document.write('window.print();\n');
				}


				if(navigator.userAgent.toLowerCase().indexOf("firefox") > -1)
				{
				printWindow.document.write('window.close();\n');
				}
				else
				{
				printWindow.document.write('chkstate();\n');
				}
				printWindow.document.write('}\n');


				printWindow.document.write('function chkstate()\n');
				printWindow.document.write('{\n');
				printWindow.document.write('if(document.readyState=="complete")');
				printWindow.document.write('{\n');
				printWindow.document.write('window.close();\n');
				printWindow.document.write('}\n');
				printWindow.document.write('else{\n');
				printWindow.document.write('setTimeout("chkstate();",3000);\n');
				printWindow.document.write('}\n');
				printWindow.document.write('}\n');

				printWindow.document.write('function printChrome()\n');
				printWindow.document.write('{\n');
				printWindow.document.write('if(document.readyState=="complete")');
				printWindow.document.write('{\n');
				printWindow.document.write('window.print();\n');
				printWindow.document.write('}\n');
				printWindow.document.write('else{\n');
				printWindow.document.write('setTimeout("printChrome();",3000);\n');
				printWindow.document.write('}\n');
				printWindow.document.write('}\n');


				printWindow.document.write('</scr');
				printWindow.document.write('ipt>');

				printWindow.document.write('</head>');
				if(document.getElementById('printFooter'))
				document.getElementById('printFooter').style.display='block';
				if(document.getElementById('buttonSet1'))
				document.getElementById('buttonSet1').style.display='none';
				printWindow.document.write('<body onload="winPrint()" style="background:#fff;margin:0px;padding:0px;">');
				printWindow.document.write('<table width="100%" style="padding:0px;margin:0px;" align="center">');
				printWindow.document.write('<tr>');
				printWindow.document.write('<td style="background-color:#fff;">');
				printWindow.document.write('<table align="center" width="100%">');
				printWindow.document.write('<tr>');
				printWindow.document.write('<td>');
				printWindow.document.write('<div style="height: 100%;">');
				printWindow.document.write(printContent.innerHTML);
				printWindow.document.write('</div>');
				printWindow.document.write('</td>');
				printWindow.document.write('</tr>');
				printWindow.document.write('</table>');
				printWindow.document.write('</td>');
				printWindow.document.write('</tr>');
				printWindow.document.write('</table>');
				printWindow.document.write('</body>');
				printWindow.document.write('</html>');
				if(document.getElementsByClassName('expand'))
				document.getElementsByClassName('expand').click();
				printWindow.document.close();
				if(document.getElementById('printFooter'))
				document.getElementById('printFooter').hide();
				if(document.getElementById('buttonSet1'))
				document.getElementById('buttonSet1').show();


				}
				</script>
				<div id="KPGpaymentReceiptDiv" align="center" style="position:absolute;z-index:9000;padding:10px;top:60%;left:2%;width:90%;border:1px dotted #333;text-align:center;height:auto;background:#fff;">
				<div align="left" style="padding-top:20px;color: #936;font-weight: normal;font-size: 19px;padding-left: 10px;padding-right: 0px;">
				<img alt="FPX" title="FPX" style="padding-right:20px;" valign="middle" width="130px" src="<?php echo plugins_url('images/logoFPX.png',__FILE__); ?>">
				Transaction receipt</div>

				<?php
				if($transaction_status=="Successful")
				{
				?>
				<div align="center" style="font-size:17px;color:#080;font-weight:bold;">Thank you,we have accepted your payment.</div>
				<?php
				}
				elseif($transaction_status=="Pending")
				{
				?>
				<div align="center" style="font-size:17px;color:#880;font-weight:bold;">Your payment is still in progress and will be completed soon.</div>
				<?php
				}
				else
				{
				?>
				<div align="center" style="font-size:17px;color:#800;font-weight:bold;">Sorry,we are unable to accept your payment.</div>
				<?php
				}
				?>

				<div class="transactionContainerContents">

				<div class="transactionSummaryDiv">
				<div align="left" style="color: #0087cb;font-size: 20px;text-decoration: underline;font-weight: normal;">Transaction details</div>
				<table width="100%">
				<tbody>
				<tr align="left" valign="top">
				<td width="50%"></td>
				<td width="50%"></td>
				</tr>
				<tr align="left" valign="top">
				<td>Transaction Status</td>
				<td style="color:<?php if($transaction_status=='Successful') echo '#080'; elseif($transaction_status=='Pending') echo '#880'; else echo '#800'; ?>;"><?php echo $transaction_status_description; ?></td>
				</tr>
				<tr align="left" valign="top">
				<td>Transaction date and time</td>
				<td><?php echo date('j F Y h:i:s A',strtotime($transaction_time)); ?></td>
				</tr>
				<tr align="left" valign="top">
				<td>FPX Transaction ID</td>
				<td><?php echo $transaction_fpx; ?></td>
				</tr>
				<tr align="left" valign="top">
				<td>Seller Order Number</td>
				<td><?php echo $order_no_post; ?></td>
				</tr>
				<tr align="left" valign="top">
				<td>Exchange Order Number</td>
				<td><?php echo $transaction_kpg; ?></td>
				</tr>
				<tr align="left" valign="top">
				<td>Buyer Bank Name</td>
				<td><?php echo $transaction_bank_name; ?></td>
				</tr>
				<tr align="left" valign="top">
				<td>Transaction Amount</td>
				<td><?php echo $transaction_currency.' '.number_format($order_amount,2,'.',','); ?></td>
				</tr>
				<tr align="left" valign="top">
				<td>Order Description</td>
				<td><?php echo $order_description; ?></td>
				</tr>
				<tr align="left" valign="top">
				<td>Seller</td>
				<td><?php echo $transaction_seller_name; ?></td>
				</tr>
				<tr align="left" valign="top">
				<td>Buyer name</td>
				<td><?php echo $order_buyer_name; ?></td>
				</tr>
				<tr align="left" valign="top">
				<td>Buyer tel. no.</td>
				<td><?php echo $order_buyer_tel; ?></td>
				</tr>
				<tr align="left" valign="top">
				<td>Buyer email</td>
				<td><?php echo $order_buyer_email; ?></td>
				</tr>
				</tbody>
				</table>
				</div>
				<div align="center">
				<p align="left" style="color:#0a0;"><img alt="Go Green" src="<?php echo plugins_url('images/gogreen.jpg',__FILE__); ?>" align="center"> Please do not print this document unless you really need to.</p>
				<b>THIS IS COMPUTER-GENERATED DOCUMENT. NO SIGNATURE IS REQUIRED.</b>
				</div>
				<div id="printFooter" align="right" style="display:none;">
				<br><i>Printed on <?php echo date('j F Y h:i:s A'); ?></i>
				</div>
				<div align="center" id="buttonSet1" class="addFormBtnSet">
				<?php
				if($transaction_status=='Unsuccessful')
				{ # Display Pay Again button for unsuccessful transaction start.
				?>
					<input id="payAgainBtn" name="payAgainBtn" type="button" value="Pay again" onclick="top.location.href='<?php echo esc_url( $order->get_checkout_payment_url() ); ?>';">
				<?php
				} # Display Pay Again button for unsuccessful transaction end.
				?>
				<input id="printBtn" name="printBtn" type="button" value="Print" onclick="CallPrint('KPGpaymentReceiptDiv','FPX Transaction Receipt <?php echo $transaction_fpx; ?>');">
				<input id="shopMoreBtn" name="shopMoreBtn" type="button" value="Shop more items" onclick="top.location.href='<?php echo get_permalink( woocommerce_get_page_id( 'shop' ) ); ?>';">
				<?php
				if(is_user_logged_in())
				{ # Display View order details and Go to My Account buttons for the logged in users start.
				?>
					<input id="viewOrderBtn" name="viewOrderBtn" type="button" value="View order details" onclick="top.location.href='<?php echo $order->get_view_order_url(); ?>';">
					<input id="gotoProfileBtn" name="gotoProfileBtn" type="button" value="Go to My Account" onclick="top.location.href='<?php echo get_permalink(get_option('woocommerce_myaccount_page_id')); ?>';">
				<?php
				} # Display View order details and Go to My Account buttons for the logged in users end.
				else
				{ # If user was a guest, just display the View order details button start.
				?>
					<input id="viewOrderBtn" name="viewOrderBtn" type="button" value="View order details" onclick="top.location.href='<?php echo esc_url( $order->get_checkout_payment_url() ); ?>';">
				<?php
				} # If user was a guest, just display the View order details button end.
				?>
				</div>

				</div>

				</div>
				<script type="text/javascript">
				document.write('<html><head></head><body>');
				document.write('<div style="position:absolute;z-index:8000;overflow:hidden;background:#fff;width:100%;height:200px;top:300px;">');
				document.write('</div>');
				document.write('</body></html>');
				</script>
				<?php
				} # Permit to process order with pending payment status end.
			} # Do only proceed on the matched hash responses and hash in the database end.
		} # Function to check the responses from KPG end.

		public function portal_key_missing_message()
		{ # Function to display error message if the KPG Portal Key is not provided start.
			$message='<div class="error">';
			$message.='<p><strong>KICT Payment Gateway</strong> Please provide your Portal key <a href="'.get_admin_url().'admin.php?page=wc-settings&tab=checkout&section=KPG">here</a>.</p>';
			$message.='</div>';
			echo $message;
		} # Function to display error message if the KPG Portal Key is not provided end.

	} # Class KPG_gateway end.

} # Function KPG_gateway_load end.
